from pkg_resources import parse_version

from . import app
from . import autostart
from . import config
from . import podman
from . import repo
from .flock import locked


class AppError(Exception):
    def __init__(self, app_name):
        super().__init__(app_name)
        self.app_name = app_name

class AppAlreadyInstalledError(AppError):
    pass

class AppNotInstalledError(AppError):
    pass

class AppNotInRepoError(AppError):
    pass

class AppNotUpdateableError(AppError):
    pass


def list_installed():
    return dict(sorted(podman.get_apps().items()))

def list_online():
    return {app:definition['version'] for app,definition in sorted(repo.get_apps().items())}

def list_updates():
    online_apps = {app:definition['version'] for app,definition in repo.get_apps().items()}
    apps = {app:f'{version} -> {online_apps[app]}' for app,version in podman.get_apps().items()
            if app in online_apps
            and parse_version(online_apps[app]) > parse_version(version)}
    return dict(sorted(apps.items()))

@locked()
def install(app_name):
    if app_name in podman.get_apps():
        raise AppAlreadyInstalledError(app_name)
    if app_name not in repo.get_apps():
        raise AppNotInRepoError(app_name)
    app.install(app_name)

@locked()
def update(app_name):
    if app_name not in podman.get_apps():
        raise AppNotInstalledError(app_name)
    if app_name not in list_updates():
        raise AppNotUpdateableError(app_name)
    app.update(app_name)

@locked()
def uninstall(app_name):
    if app_name not in podman.get_apps():
        raise AppNotInstalledError(app_name)
    app.uninstall(app_name)

@locked()
def start(app_name):
    if app_name not in podman.get_apps():
        raise AppNotInstalledError(app_name)
    podman.start_pod(app_name)

@locked()
def stop(app_name):
    if app_name not in podman.get_apps():
        raise AppNotInstalledError(app_name)
    podman.stop_pod(app_name)

def status(app_name=None):
    if app_name is not None and app_name not in podman.get_apps():
        raise AppNotInstalledError(app_name)
    return podman.get_pod_status(app_name)

@locked()
def set_autostart(app_name, enabled):
    if app_name not in podman.get_apps():
        raise AppNotInstalledError(app_name)
    autostart.set_app(app_name, enabled)

@locked()
def start_autostarted():
    for app_name in autostart.get_apps():
        podman.start_pod(app_name)

@locked()
def stop_all():
    for app_name in podman.get_apps():
        podman.stop_pod(app_name)

@locked()
def login(host, username, password):
    config.write_auth(host, username, password)
    repo.load(force=True)

@locked()
def prune():
    podman.prune()
